/**
 * 命令执行引擎
 */
const EngineContext = require("./EngineContext");

class Engine {
	constructor(taskList) {
		this.index = 0;
		this.context = new EngineContext();
		this.taskList = taskList;
	}

	getContext(){
		return this.context;
	}

	handleError(err) {
		return new Promise((resolve, reject) => {
			let promises = [];
			for (let i = 0; i < this.index; i++) {
				let task = this.taskList[i];
				let promise = task.handleError(this.context);
				promises.push(promise);
			}
			Promise.all(promises).then(() => {
				resolve(err);
			}).catch(e => {
				resolve(e);
			});
		});
	}

	start() {
		return new Promise((resolve, reject) => {
			if (this.index < this.taskList.length) {
				let task = this.taskList[this.index];
				try {
					let promise = task.exe(this.context);
					promise.then(() => {
						this.index++;
						return this.start();
					}).then(() => {
						resolve();
					}).catch(err => {
						this.handleError(err).then(e => {
							reject(e);
						});
					});
				} catch (err) {
					this.handleError(err).then(e => {
						reject(e);
					});
				}
			} else {
				let promises = [];
				for (let i = 0, l = this.taskList.length; i < l; i++) {
					let task = this.taskList[i];
					let promise = task.beforeDone(this.context);
					promises.push(promise);
				}
				Promise.all(promises).then(() => {
					resolve();
				}).catch(err => {
					this.handleError(err).then(e => {
						reject(e);
					});
				});
			}
		});
	}
}

module.exports = Engine;