/**
 * 同步服务器上的插件
 */
const ITask = require("../ITask");
const configUtils = require('../../../../utils/ConfigUtils');
const fileUtils = require("../../../../utils/FileUtils");
const constant = require("../../../../utils/Constant");
const pathUtils = require("../../../../utils/PathUtils");
const needle = require("needle");
const VPlatformPluginInstaller = require('v3-installer/src/installer/VPlatformPluginInstaller');

class SyncTask extends ITask {

	constructor(params) {
		this.pluginsDir = null;
	}

	exe(ctx) {
		return new Promise((resolve, reject) => {
			if (this.validateConfig()) {
				let platformCfg = configUtils.getV3DevConfig();
				let server = platformCfg.server;
				let url = server + constant.PLUGIN_DOWNLOAD_URL;
				let dir = fileUtils.getV3PluignTempRelativeDir();
				this.pluginsDir = pathUtils.toPluginPath(dir);
				let output = fileUtils.toV3PluginTempRelativePath(dir);
				let absPath = pathUtils.toPluginPath(output);
				fileUtils.makeDir(pathUtils.getParentPath(absPath));
				let task = this;
				needle.get(url, {
					output: absPath
				}, (err, resp, body) => {
					if(err){
						reject(err);
						return;
					}
					try {
						let installer = new VPlatformPluginInstaller(absPath);
						let promise = installer.install();
						promise.then(()=>{
							resolve();
						}).catch(e=>{
							reject(e);
						});
					} catch (e) {
						reject(e);
					}
				});
			}
		});
	}
	/**
	 * 处理异常
	 * @param {EngineContext} ctx 引擎上下文
	 */
	handleError(ctx) {
		return new Promise((resolve, reject) => {
			try {
				if (this.pluginsDir) {
					fileUtils.removeDirectory(this.pluginsDir);
				}
				resolve();
			} catch (err) {
				reject(err);
			}
		});

	}
	/**
	 * 校验配置信息
	 */
	validateConfig() {
		let platformCfg = configUtils.getV3DevConfig();
		if (!platformCfg) {
			console.warn("WARN: package.json文件中没有配置v3Platform属性，同步V3平台插件将终止！");
			return false;
		}
		let server = platformCfg.server;
		if (!server) {
			console.warn("WARN: package.json文件中v3Platform没有配置V3服务地址[server]，同步V3平台插件将终止！");
			return false;
		}
		return true;
	}

}

module.exports = SyncTask;