/**
 * @author xujw 2018-8-21 14:23:57
 * 在部署过程中对执行系统缺失的vjs依赖进行补充安装
 */
const ITask = require("../ITask");
const needle = require('needle');
const VpluginDownloader = require('v3-installer/src/installer/VpluginDownloader');
const configUtils = require('../../../../utils/ConfigUtils');
const fileUtils = require('../../../../utils/FileUtils');
const PluginUpdater = require("../../http/PluginUpdater");
class VjsPatchTask extends ITask {
    exe(ctx) {
        return new Promise((resolve, reject) => {
            console.log('获取执行系统vjs依赖信息...');
            let jar = ctx.get("jar");
            let v3cfg = jar.v3DevConfig;
            if (!v3cfg) {
                reject("读取.v3devrc配置文件失败!");
                return;
            }
            let depVjs = v3cfg.depVjsList || [];
            let vjsNameList = depVjs.map(vjs => {
                return vjs.name;
            });
            this.getUninstallVjs(v3cfg.server, vjsNameList).then(vjsList => {
                if (vjsList.length == 0) {
                    resolve();
                    return;
                }
                console.log('检测到执行系统vjs依赖缺失,开始安装缺失依赖...');
                let downloadList = this.getDepInfoByVjsList(v3cfg, vjsList);
                let downloader = new VpluginDownloader(v3cfg);
                downloader._addTask(downloadList);
                downloader._start().then(jarfiles => {
                    let promises = [];
                    let symbolicNameList = Object.keys(jarfiles);
                    symbolicNameList.forEach(syName => {
                        console.log(`正在安装${syName} ...`);
                        let updaterPromise = new PluginUpdater({
                            "jar": jarfiles[syName],
                            "packageJson": configUtils.getPackageConfig(),
                            "symbolicName": syName
                        });
                        promises.push(updaterPromise.update());
                    });
                    Promise.all(promises).then(() => {
                        console.log('执行系统vjs依赖安装完成!');
                        fileUtils.removeDirectory(downloader.downloadPath);
                        resolve();
                        return;
                    });
                })
            })
        });
    }
    /**
     * 给定一组vjs,获取其中执行系统未安装的vjs列表
     * @param {*} server 执行系统地址
     * @param {*} vjsNameList 
     */
    getUninstallVjs(server, vjsNameList) {
        return new Promise((resolve, reject) => {
            needle.post(`${server}/module-operation!executeOperation?operation=Manager&action=VjsExists&vjsList=${JSON.stringify(vjsNameList)}`, {
                timeout: 180000
            }, (err, resp, body) => {
                if (err) {
                    reject(err);
                    return;
                }
                if (body) {
                    try {
                        body = JSON.parse(body);
                    } catch (err) {
                        console.log('读取执行系统vjs信息失败,已跳过!');
                        resolve([]);
                    }
                    if (!body.success) {
                        reject(body.msg);
                        return;
                    }
                    resolve(this.getUnexistVjs(vjsNameList, body.data));
                    return;
                }
            });
        });
    }
    /**
     * 比较本地vjs和服务端vjs找出服务端未安装的vjs依赖
     * @param {*} localVjs 
     * @param {*} serverVjs 
     */
    getUnexistVjs(localVjs, serverVjs) {
        let unexistVjs = [];
        localVjs.forEach(lVjs => {
            let exist = false;
            serverVjs.forEach(sVjs => {
                if (lVjs == sVjs) {
                    exist = true;
                }
            });
            if (!exist) {
                unexistVjs.push(lVjs);
            }
        });
        return unexistVjs;
    }
    /**
     * 通过vjs获取该vjs在配置文件中的依赖信息,symbolicName,version,libType等信息
     * @param {*} unexistVjs 
     */
    getDepInfoByVjsList(v3cfg, unexistVjs) {
        //获取全部依赖信息
        let dependencies = v3cfg.dependencies.concat(v3cfg.require);
        let depVjs = v3cfg.depVjsList;
        let symbolicNameList = [],
            depInfo = [];
        for (let i = 0; i < unexistVjs.length; i++) {
            depVjs.forEach(vjs => {
                if (vjs.name == unexistVjs[i]) {
                    symbolicNameList.push(vjs.symbolicName);
                }
            });
        }
        symbolicNameList.forEach(syName => {
            dependencies.forEach(dep => {
                if (dep.name == syName) {
                    depInfo.push(dep);
                }
            })
        });
        return depInfo;
    }
}

module.exports = VjsPatchTask;