const needle = require("needle");
const constant = require("../../../utils/Constant");
const stringUtils = require("../../../utils/StringUtils");
const widgetMavenUtils = require('../../utils/WidgetMavenUtils');
const fileUtils = require("../../../utils/FileUtils");
const configUtils = require('../../../utils/ConfigUtils');
const fs = require("fs");

class PluginDeployer{

	constructor(params){
		this.jar = params.jar;
		this.packageJson = params.packageJson;
		this.v3DevConfig = params.v3DevConfig;
		this.source = params.source;
		this.manifest = params.manifest;
	}
	
	deploy(){
		return new Promise((resolve,reject)=>{
			let v3Platform = this.packageJson.v3Platform;
			let promise = configUtils.getVerification();
			promise.then(result=>{
				let account = result.account,pwd = result.pwd;
				let attributeExtend = [{
					"key": "pluginType",
					"value": this.v3DevConfig.type
				}, {
					"key": "devType",
					"value": "extension"
				}, {
					"key": "devEnv",
					"value": "nodejs"
				}];
				let pluginCode = this.v3DevConfig.pluginCode;
				if(Array.isArray(pluginCode)){
					pluginCode.forEach(code => {
						attributeExtend.push({
							"key": "pluginCode",
							"value": code
						});
					});
				}else{
					attributeExtend.push({
						"key": "pluginCode",
						"value": pluginCode
					});
				}
				let keywords = this.packageJson.keywords;
				if(keywords){
					if(Array.isArray(keywords)){
						keywords.forEach(keyword => {
							attributeExtend.push({
								"key": "keyword",
								"value": keyword
							});
						});
					}else{
						attributeExtend.push({
							"key": "keyword",
							"value": keywords
						});
					}
				}
				let params = {
					"data" : {
						"groupId":constant.DEFAULT_GROUP_ID,
						"artifactId":widgetMavenUtils.getArtifactId(),
						"version":this.manifest.getHeader("Bundle-Version"),
						"symbolic-name":this.manifest.getHeader("Bundle-SymbolicName"),
						"bundle-name":this.packageJson.name,
						"type":"RuntimeJava",
						"import-package":"",
						"export-package":"",
						"lastModified":this.manifest.getHeader("Bnd-LastModified"),
						"description":this.packageJson.description,
						"owner":account,
						"password":stringUtils.toMd5(pwd),
						"provider":account,
						"partLibCode":this.v3DevConfig.libType ? this.v3DevConfig.libType:"dev",
						"attributeExtend": attributeExtend
					}
				};
				let json = JSON.stringify(params);
				let encodedJson = encodeURIComponent(json);
				let data = {
					token : encodedJson,
					ajaxRequest : true
				};
				data[fileUtils.getFileName(this.jar)] = fs.readFileSync(this.jar);
				data[fileUtils.getFileName(this.source)] = fs.readFileSync(this.source);
				needle.post(constant.DEPLOY_URL, data, {
					timeout:180000,
					multipart: true 
				},(err, resp, body)=> {
					/* 判断部署是否出错 */
					if(body.isError){
						return reject({message:body.msg});
					}
					if(err){
						return reject(err);
					}
					return resolve(resp,body);
				});
			});
		});
	}
}

module.exports = PluginDeployer;