const needle = require("needle");
const fileUtils = require('../../../utils/FileUtils');
const stringUtils = require('../../../utils/StringUtils');
const Constant = require('../../../utils/Constant');
const configUtils = require('../../../utils/ConfigUtils');
const fs = require("fs");

class PluginUpdater {
	constructor(params) {
		this.jar = params.jar;
		this.manifest = params.manifest;
		this.packageJson = params.packageJson;
		this.symbolicName = params.symbolicName;
	}
	update() {
		return new Promise((resolve, reject) => {
			configUtils.getVerification().then(result => {
				return new Promise((rs, rj) => {
					configUtils.getServer().then(info => {
						rs({
							account: result.account,
							pwd: result.pwd,
							server: info.server
						});
					});
				});
			}).then(data => {
				let account = data.account,
					pwd = data.pwd,
					server = data.server;
				let params = {
					"command": "updateBundleToDebug",
					"symbolicName": this.manifest ? this.manifest.getHeader("Bundle-SymbolicName") : this.symbolicName,
					"fileName": fileUtils.getFileName(this.jar)
				};
				params[fileUtils.getFileName(this.jar)] = fs.readFileSync(this.jar);
				let basicUsernameAndPwd = account + ":" + pwd;
				needle.post(this.getUpdateUrl(server), params, {
					timeout: 180000,
					multipart: true,
					headers: {
						"Authorization": "Basic " + stringUtils.toBase64(basicUsernameAndPwd)
					}
				}, (err, resp, body) => {
					if (err) {
						/* 如果出错判断错误码并转换为对应的错误信息 */
						switch (err.code) {
							case 'ECONNREFUSED':
								err.message = '与执行系统[' + err.address + ':' + err.port + ']连接失败,请确认执行系统已启动!';
								break;
							case 'ETIMEDOUT':
								err.message = '与执行系统[' + err.address + ':' + err.port + ']连接超时,请重试!';
								break;
						}
						return reject(err);
					}
					// 如果没有返回异常信息则再判断http状态码
					if (resp.statusCode != 200) {
						switch (resp.statusMessage) {
							case 'Unauthorized':
								return reject('认证失败,vstore账号或密码错误,请在.v3devrc配置文件中修改account/pwd字段!');
							default:
								return reject(`构件安装失败,服务器状态码[${resp.statusCode}],状态码信息:${resp.statusMessage}.`);
						}
					}
					return resolve(resp, body);
				});
			});
		});
	}

	getUpdateUrl(server) {
		var servlet = Constant.BUNDLE_UPDATE_SERVLET;
		if (server.endsWith('/') && servlet.startsWith('/')) {
			servlet = servlet.substring(1);
		}
		return server + servlet;
	}

}

module.exports = PluginUpdater;