
const objectUtils = require('../../../utils/ObjectUtils');

class Instance{ 

	constructor(params){
		this.component = params.component;
		this.properties = params.properties;
	}

	getProperty(name){
		var pros = this.properties;
		if(pros&&pros.hasOwnProperty(name)){
			return pros[name];
		}
		return null;
	}

	setProperty(name,val){
		var pros = this.properties;
		if(!pros){
			pros = {};
			this.properties = pros;
		}
		pros[name] = val;
	}

	_toArrayValStr(val){
		let buff = [];
		for(let i=0,l=val.length;i<l;i++){
			let item = val[i];
			buff.push("property { ");
			buff.push(this._toValStr(item));
			buff.push("}");
		}
		return buff.join("");
	}

	_toMapValStr(val){
		let buff = [];
		for(let name in val){
			let keyVal = val[name];
			buff.push("property { ");
			buff.push("$name=\"");
			buff.push(name);
			buff.push("\" ");
			buff.push(this._toValStr(keyVal));
			buff.push("}");
		}
		return buff.join("");
	}

	_toStrValStr(val){
		let buff = [];
		buff.push("$value=\"");
		let json = JSON.stringify([val]);
		let valStr = json.substring(2, json.length-2);
		buff.push(valStr);
		buff.push("\" ");
		return buff.join("");
	}

	_toValStr(val){
		let buff = [];
		if(objectUtils.isArray(val)){
			buff.push("$type=\"list\" ");
			buff.push(this._toArrayValStr(val));
		}else if(objectUtils.isObject(val)){
			buff.push("$type=\"map\" ");
			buff.push(this._toMapValStr(val));
		}else{
			buff.push(this._toStrValStr(val));
		}
		return buff.join("");
	}

	_toValueStr(key,val){
		let buff = [];
		buff.push("property { $name=\"");
		buff.push(key);
		buff.push("\" ");
		buff.push(this._toValStr(val));
		buff.push("}");
		return buff.join("");
	}

	toString(){
		let buff = ["instance { $component=\""];
		buff.push(this.component);
		buff.push("\" ");
		if(this.properties){
			for(let attr in this.properties){
				let val = this.properties[attr];
				buff.push(this._toValueStr(attr,val));
			}
		}
		buff.push("}");
		return buff.join("");
	}
}

module.exports = Instance;
/*
var i = new Instance({
	component:"widget.extension",
	properties:{
		symbolicName:"vjs.framework.extension.platform.custom.widget.element-tree",
		vjsNames:["vjs.framework.extension.platform.custom.widget.vuiTreeExtra"],
		resources:["itop/common/extension/custom/widget/element-tree/tree.min.js"],
		componentDefines:[
			{
				componentName:"vuiSwiper",
				dataProp:"data",
				dataType:"Tree"
			}
		]
	}
});
console.log(i.toString());*/