const archiver = require('archiver');
const Resource = require('./Resource');
const fs = require('fs-extra');
const fileUtils = require("../../../utils/FileUtils");
const pathUtils = require('../../../utils/PathUtils');
const PathSep = require('path').sep;
class Jar {

	constructor(params) {
		this.manifest = params.manifest;
		this.resources = params.resources || [];
	}

	appendResource(resource) {
		this.resources.push(resource);
	}

	persistence(absPath) {
		return new Promise((resolve, reject) => {
			let res = this.resources.concat();
			if (this.manifest) {
				let resource = new Resource({
					content: this.manifest.toString(),
					dist: 'META-INF/MANIFEST.MF'
				});
				res.push(resource);
			} else {
				reject(new Error("Jar文件生成失败，原因：没有定义Manifest信息！"));
			}
			fileUtils.makeDir(pathUtils.getParentPath(absPath));
			let tempPath = pathUtils.getParentPath(absPath) + PathSep + "temp__";
			let output = fs.createWriteStream(absPath);
			let archive = archiver('zip', {
				zlib: {
					level: 9
				}
			});
			output.on('close', function () {
				fs.removeSync(tempPath);
				resolve(archive);
			});
			archive.on('error', reject);
			archive.pipe(output);
			for (let i = 0, l = res.length; i < l; i++) {
				let resource = res[i];
				let dist = resource.getDist();
				let absPath = tempPath + PathSep + dist;
				resource.setDist(absPath);
				resource.persistence();
			}
			archive.directory(tempPath, false);
			archive.finalize();
		});
	}
}

module.exports = Jar;