const Jar = require("./Jar");
const FileResource = require("./FileResource");
const widgetPathUtils = require('../../utils/WidgetPathUtils');
const pathUtils = require("../../../utils/PathUtils");
const V3PluginFileResource = require("./V3PluginFileResource");
const xml2js = require("xml2js");
const Resource = require("./Resource");
const Instance = require("./Instance");
const Mainfest = require('./V3PlatformManifest');
const fileUtils = require("../../../utils/FileUtils");
const constant = require("../../../utils/Constant");
const widgetMavenUtils = require('../../utils/WidgetMavenUtils');
const Vjs = require("./vjs/Vjs");
const Module = require("./vjs/Module");
const VjsToXml = require("./vjs/VjsToXml");
const Service = require("./vjs/Service");
const Dependency = require("./vjs/Dependency");
const Implement = require("./vjs/Implement");
const configUtils = require('../../../utils/ConfigUtils');
const stringUtils = require('../../../utils/StringUtils');
const path = require('path');
const fs = require('fs');
const PackageTask = require('../engine/task/PackageTask');
const skinInfoUtil = require('../../utils/SkinInfoUtil');
const CssScriptResScanner = require('../../utils/CssScriptResScanner');

class V3PlatformJar {

	constructor(params) {
		this.packageJson = params.packageJson;
		/* npm打包后的tgz文件路径 */
		this.tarballPath = '';
		this.sourceExclude = ["node_modules", ".vscode", ".svn", ".idea"];
		this.v3DevConfig = params.v3DevConfig;
		this.version = params.v3DevConfig.version;
		this.libs = params.libs;
		this.components = params.components;
		this.dependencies = params.dependencies;
		this.implements = params.v3DevConfig.implements;
		this.dist = null;
		this.sourceDist = null;
		this.manifest = null;
		this.jar = null;
		this.sourceJar = null;
		this.context = params.context;
		// 判断是否需要创建皮肤信息
		if (this.v3DevConfig.skin && Object.keys(this.v3DevConfig.skin).length > 0) {
			this.createSkinInfo = true;
		}
	}
	_createTagVjs(componentNames, pagePaths) {
		/* 保存本项目的VJS列表 */
		let vjsList = [];
		/* 读取配置文件中的依赖信息,将配置文件中依赖的vjs添加到vjslist */
		let v3Cfg = configUtils.getV3DevConfig();
		let deps = v3Cfg && v3Cfg.dependencies && v3Cfg.dependencies.map(dep => {
			if(dep.name.startsWith("vjs.framework.extension.platform.custom.resource")){
				return dep.name.substring(dep.name.lastIndexOf('.') + 1);
			}else{//依赖平台的vjs
				return dep.name;
			}
		});
		// 统一添加v3-vdk依赖
		if (v3Cfg.pluginCode !== 'v3-vdk') {
			deps.push('v3-vdk');
		}
		componentNames.forEach(cmpName => {
			let resourcesScript=[];
			if(pagePaths&&pagePaths.length>0){
				resourcesScript.push('[');
				pagePaths.forEach(item=>{
					resourcesScript.push('\"');
					if(typeof(item) == "object" && !!item.cached){
						let path = item.path;
						resourcesScript.push(path);
						if(path.indexOf('?')!=-1){
							resourcesScript.push('&v=');
						}else{
							resourcesScript.push('?v=');
						}
						resourcesScript.push((item.hash+'').replace('-','_'));
						resourcesScript.push('\"');
					}else{
						let path = item;
						if(typeof(item) != "string"){
							path = item.path;
						}
						resourcesScript.push(path);
						if(path.indexOf('?')!=-1){
							resourcesScript.push('&v=');
						}else{
							resourcesScript.push('?v=');
						}
						resourcesScript.push('\"');
						resourcesScript.push('+((new Date()).getTime())');
					}
					resourcesScript.push(',');
				});
				resourcesScript.pop();
				resourcesScript.push(']');
			}else{
				resourcesScript.push("[]");
			}
			let beforeScript=`vdk.resource.add(new vdk.resource({"id":"${this.v3DevConfig.pluginCode}","paths":${resourcesScript.join('')},"deps":${JSON.stringify(deps)}}));`;
			let factory = `function(require,exports,module){}`;
			let modules = [new Module({
				id: widgetMavenUtils.toModuleId(widgetMavenUtils.getTagResourceVjsName(), "./Resource", widgetMavenUtils.getMavenVersion()),
				name: "./Resource",
				factory: factory,
				beforeScript:beforeScript
			})];
			let impls = null;
			if(this.implements){
				impls = [];
				for(let i=0,l=this.implements.length;i<l;i++){
					impls.push(new Implement(this.implements[i]));
				}
			}
			let vjs = new Vjs({
				name: widgetMavenUtils.getTagResourceVjsName(cmpName),
				version: widgetMavenUtils.getMavenVersion(),
				implements : impls,
				dependencies: [
					new Dependency({
						name: "vjs.framework.extension.platform.custom.resource.v3-vdk",
						version: null
					})
				],
				services: [],
				modules: modules
			});
			if (this.createSkinInfo || this.context.skinInfo) {
				vjs.dependencies.push(new Dependency({
					name: vjs.getName() + '.$skin',
					version: null
				}));
			}
			/* 读取配置文件中的依赖信息,将配置文件中依赖的vjs添加到vjslist */
			let v3Cfg = configUtils.getV3DevConfig();
			if (v3Cfg && v3Cfg.depVjsList) {
				for (let i = 0; i < v3Cfg.depVjsList.length; i++) {
					vjs.dependencies.push(new Dependency({
						name: v3Cfg.depVjsList[i].name
					}));
				}
			}
			vjsList.push(vjs);
		});
		let skinVjs = skinInfoUtil.getSkinVjs(this.v3DevConfig, this.context.skinInfo);
		if (skinVjs) {
			vjsList = vjsList.concat(skinVjs);
		}
		return vjsList;
	}

	/**
	 * 解析libs中的资源，分析出其依赖的其他资源
	 * 例：在css文件中使用url方式引入其他资源
	 */
	_parseLibResources(){
		return new Promise((resolve,reject)=>{
			let promises = [];
			this.libs.forEach((obj)=>{
				let path = obj.path;
				let fileName = fileUtils.getFileName(path);
				if(fileName.endsWith('css')){
					let scanner = new CssScriptResScanner(path);
					let promise = scanner.scan();
					promises.push(promise);
				}
			});
			Promise.all(promises).then(res=>{
				let resList = [],resObj = [];
				res.forEach(item=>{
					resList = resList.concat(item);
				});
				resList.forEach(item=>{
					resObj.push({
						path : item,
						cached: true
					});
				});
				resolve(resObj);
			}).catch(err=>{
				reject(err);
			});
		});
	}

	_getPaths(resLibs){
		let res = [];
		resLibs.forEach(item=>{
			res.push(item.path);
		});
		return res;
	}

	_init() {
		return new Promise((resolve, reject) => {
			let promise = this._parseLibResources();
			promise.then(res=>{
				let excludeReg = /\.(png|jpe?g|gif|svg|woff|woff2|eot|ttf|map)(\?\S*)?$/;
				let resources = [];
				let pagePaths = this.libs.concat(res);
				let dealLibs = fileUtils.removeDuplicateDir(this._getPaths(pagePaths)),
					distLibs = [],
					hostPaths = [];
				// 添加外部url资源
				try {
					let urlSources = configUtils.getUrlSourcesByCfg(this.v3DevConfig);
					hostPaths = hostPaths.concat(urlSources.css, urlSources.js);
				} catch (error) {
					reject(error);
					return;
				}
				pagePaths.forEach(pageObj=>{
					let pagePath = pageObj.path;
					let file = new V3PluginFileResource({
						name: this.v3DevConfig.pluginCode,
						source: pagePath,
						dist: dealLibs[pagePath]
					});
					let dist = file.getDist();
					distLibs.push(dist);
					if(this.libs.indexOf(pageObj)!=-1){
						let jarPath = dist.substring(15);
						if (!excludeReg.test(path.extname(jarPath))) {
							let ph = jarPath.replace(/\\/g, '/');
							let hash = null;
							//如果资源需要缓存，则计算出资源hashCode
							if(pageObj.cached){
								let content = fs.readFileSync(pagePath);
								hash = stringUtils.toHashCode(content.toString());
							}
							hostPaths.push({
								path : ph,
								hash:hash,
								cached : pageObj.cached
							});
						}
					}
					resources.push(file);
				});
				let xmlDefine = {};
				xmlDefine.components = {};
				xmlDefine.components.component = this.components;
				xmlDefine.dependencies = {};
				xmlDefine.dependencies.dependency = this.dependencies;
				xmlDefine.components.component.forEach(cmp => {
					cmp.scope = this.v3DevConfig.scope;
				});
				/* 添加插件信息 */
				xmlDefine.pluginType = this.v3DevConfig.type;
				xmlDefine.pluginName = this.v3DevConfig.pluginCode;
				xmlDefine.pluginVersion = this.v3DevConfig.version;
				xmlDefine.dependencies = {};
				xmlDefine.dependencies.dependency = this.dependencies;
				xmlDefine.sources = {};
				xmlDefine.sources.lib = distLibs;
				let builder = new xml2js.Builder();
				let xml = builder.buildObject(xmlDefine);
				let xmlResource = new Resource({
					content: xml,
					dist: constant.PLUGIN_METADATA_NAME
				});
				resources.push(xmlResource);
				let componentNames = [];
				/* 获取全部组件名称如果没有,则默认使用用户项目配置的pluginCode属性值 */
				this.components.forEach(component => {
					componentNames.push(component.getName());
				});
				if (componentNames.length == 0) {
					componentNames.push(this.v3DevConfig.pluginCode);
				}
				let vjsList = this._createTagVjs(componentNames, hostPaths);
				if (!this.v3DevConfig.VjsList) {
					this.v3DevConfig.VjsList = [];
				}
				for (let i = 0; i < vjsList.length; i++) {
					this.v3DevConfig.VjsList.push({
						'name': vjsList[i].name,
						'version': vjsList[i].version
					})
				}
				let vjsNames = [];
				for (let i = 0, l = vjsList.length; i < l; i++) {
					let vjs = vjsList[i];
					vjsNames.push(vjs.getName());
				}
				let vjsXml = new VjsToXml({
					vjsList: vjsList
				});
				let xmlStr = vjsXml.toXML();
				resources.push(new Resource({
					content: xmlStr,
					dist: "vjs/package/manager/vjsDefineXml-dev.xml"
				}));
				resources.push(new Resource({
					content: xmlStr,
					dist: "vjs/package/manager/vjsDefineXml-prd.xml"
				}));
				let skinResPath = this._addSkinResource(resources);
				// 保存生成的vjs信息到配置文件
				configUtils.persistenceV3DevCfg();
				// 使用npm打包出tgz包
				new PackageTask().exe().then(tgzPath => {
					this.tarballPath = tgzPath;
					resources.push(new FileResource({
						source: tgzPath,
						dist: widgetPathUtils.toPluginTGZSettingPath(fileUtils.getFileName(tgzPath))
					}));
					let args = this.context.get("args");
					let isCloudMode = args[0]=="cloud-mode";
					this.manifest = new Mainfest({
						isCloudMode: isCloudMode,
						packageJson: this.packageJson,
						v3DevConfig: this.v3DevConfig,
						instances: this._createIpojo(vjsNames, hostPaths, skinResPath)
					});
					this.jar = new Jar({
						manifest: this.manifest,
						resources: resources
					});
					this.sourceJar = new Jar({
						manifest: this.manifest,
						resources: this._getSourceFiles()
					});
					let outDir = this._getOutDir();
					let version = this.manifest.getHeader("Bundle-Version");
					version = version.replace(".SNAPSHOT", "-SNAPSHOT");
					this.dist = widgetPathUtils.getJarAbsPath(version);
					this.sourceDist = widgetPathUtils.getSourceJarAbsPath(version);
					return resolve();
				}).catch(err => {
					reject(err);
				})
			}).catch(err=>{
				reject(err);
			});
		})
	}
	/**
	 * 创建ipojo实例字符串
	 */
	_createIpojo(vjsNames, pagePaths, skinResPath) {
		let ipojos = [];
		let instance = new Instance({
			component: "widget.extension",
			properties: {
				symbolicName: widgetMavenUtils.getArtifactId(),
				vjsNames: vjsNames,
				resources: pagePaths,
				dependencies: this.dependencies
			}
		});
		ipojos.push(instance);
		if(skinResPath){
			let catalog = skinInfoUtil.createCatalogIPojoInst(this.v3DevConfig);
			ipojos.push(catalog);
			let skinVarPackage = skinInfoUtil.createSkinItemPackageIPojoInst(this.v3DevConfig,catalog,skinResPath);
			ipojos.push(skinVarPackage);
		}
		/*let skinIpojos = skinInfoUtil.getSkinInstance(this.v3DevConfig);
		if (skinIpojos && skinIpojos.length > 0) {
			ipojos = ipojos.concat(skinIpojos);
		}
		let widgetSkinPojo = skinInfoUtil.getWigetSkinPojo(this.v3DevConfig, skinResPaths);
		if (widgetSkinPojo && widgetSkinPojo.length > 0) {
			ipojos = ipojos.concat(widgetSkinPojo);
		}*/
		return ipojos;
	}
	/**
	 * 添加皮肤资源信息
	 */
	_addSkinResource(resources) {
		if (!this.context.skinInfo || !this.context.skinInfo.skinVars) {
			return;
		}
		const skinResDir = 'itop/form/theme/' + this.v3DevConfig.VjsList[0].name;
		let skinVars = this.context.skinInfo.skinVars;
		let content = skinInfoUtil.skinVarToXml(this.v3DevConfig, skinVars);
		let settingPath = `${skinResDir}/` + stringUtils.toMd5(skinResDir) + '.xml';
		let distPath = `resources/setting/${settingPath}`;
		resources.push(new Resource({
			content: content,
			dist: distPath
		}));
		return settingPath;
	}

	//获取jar输出目录
	_getOutDir() {
		let platformCfg = this.v3DevConfig;
		return platformCfg.out ? platformCfg.out : constant.DEFAULT_PLUGIN_DIR;
	}

	_getDistDir() {
		let platformCfg = this.v3DevConfig;
		return platformCfg.sources;
	}

	//获取源码资源
	_getSourceFiles() {
		let dir = pathUtils.getPluginPath();
		let excludes = this.sourceExclude;
		excludes.push(this._getOutDir());
		let sources = this._getDistDir();
		if (Array.isArray(sources)) {
			excludes = excludes.concat(sources);
		} else {
			excludes.push(sources);
		}
		for (let i = 0, l = excludes.length; i < l; i++) {
			excludes[i] = pathUtils.toPluginPath(excludes[i]);
		}
		let files = fileUtils.listFiles(dir, true, excludes);
		let resources = [];
		for (let i = 0, l = files.length; i < l; i++) {
			let file = files[i];
			if (fileUtils.isFile(file)) {
				let dist = file.substring(dir.length + 1);
				let resource;
				if ("package.json" == dist) {
					let platformCfg = this.packageJson.v3Platform;
					if (platformCfg) {
						platformCfg.account = null;
						platformCfg.pwd = null;
					}
					resource = new Resource({
						content: JSON.stringify(this.packageJson, null, "\t"),
						dist: dist
					});
				} else if (dist == this.packageJson.name + "-" + this.packageJson.version + ".tgz") {
					continue;
				} else {
					resource = new FileResource({
						source: file,
						dist: dist
					});
				}
				resources.push(resource);
			}
		}
		return resources;
	}

	persistence(callback) {
		return new Promise((resolve, reject) => {
			// 生成jar包前先清空target目录
			fileUtils.removeDirectory(path.dirname(this.dist));
			let promise = this.jar.persistence(this.dist);
			promise.then((archive) => {
				return this.sourceJar.persistence(this.sourceDist);
			}).then((archive) => {
				resolve(archive);
			}).catch((err) => {
				reject(err);
			});
		});
	}

	getDist() {
		return this.dist;
	}

	getSource() {
		return this.sourceDist;
	}

	getManifest() {
		return this.manifest;
	}

	getJar() {
		return this.jar;
	}

}

module.exports = V3PlatformJar;