const Manifest = require('./Manifest');
const widgetMavenUtils = require('../../utils/WidgetMavenUtils');
const os = require("os");
const constant = require("../../../utils/Constant");
const prefix = constant.DEFAULT_GROUP_ID + "-" + constant.ARTIFACT_PREFIX;

class V3PlatformManifest extends Manifest {
	
	constructor(params) {
		super(params.headers);
		this.isCloudMode = params.isCloudMode;//是否为云项目
		this.packageJson = params.packageJson;
		this.v3DevConfig = params.v3DevConfig;
		this.name = params.name || this.packageJson.name;
		this.version = params.v3DevConfig.version || this.packageJson.version;
		this.instances = params.instances;
		this._init();
	}

	_init() {
		this.setHeader("Bundle-Name", this.name);
		this.setHeader("Bundle-SymbolicName", widgetMavenUtils.getSymbolicName());
		this.setHeader("Bundle-Version", widgetMavenUtils.getMavenVersion(this.isCloudMode));
		this.setHeader("Manifest-Version", "1.0");
		this.setHeader("Built-By", os.userInfo().username);
		this.setHeader("Bnd-LastModified", (new Date()).getTime() + '');
		//此处不要改Nodejs Plugin标识，执行系统根据此标识识别构件，做热更新处理
		this.setHeader("Created-By", "Nodejs Plugin & v3-pack-cli 1.0.0");
		this.setHeader("Bundle-ManifestVersion", "2");
		this.setHeader("MatchCode", "CustomPlugins." + this.name);
		this.setHeader("MatchVersion", this.getMatchVersion());
		this.setHeader("MinMatchVersion", this.getMinMatchVersion());
		this.setHeader("VJS-Package", '[{"BundleName":null,"Builder":{"configs":{"test":"vjs/package/manager/vjsDefineXml-dev.xml","dev":"vjs/package/manager/vjsDefineXml-dev.xml","prd":"vjs/package/manager/vjsDefineXml-prd.xml"},"builderName":"vjs.builder.implementation.runtime.xmlparser"},"Dependencies":{},"Provides":[]}]');
		if (this.instances && this.instances.length > 0) {
			let buff = [];
			for (let i = 0, l = this.instances.length; i < l; i++) {
				let instance = this.instances[i];
				buff.push(instance.toString());
			}
			this.setHeader("iPOJO-Components", buff.join(""));
		}
	}

	getMatchVersion() {
		return this.v3DevConfig.matchVersion ? this.v3DevConfig.matchVersion : 1;
	}

	getMinMatchVersion() {
		return this.v3DevConfig.minMatchVersion ? this.v3DevConfig.minMatchVersion : 1;
	}
}

module.exports = V3PlatformManifest;