/**
 * 皮肤资源构件ipojo实体生成
 * @author xuwj 2018-12-7
 */
const path = require('path');
const Instance = require('../jar/model/Instance');
const widgetPathUtils = require('./WidgetPathUtils');
const xml2js = require("xml2js");
const Vjs = require('../jar/model/vjs/Vjs');
const Dependency = require("../jar/model/vjs/Dependency");
const Module = require("../jar/model/vjs/Module");
const idPrefix = 'theme.catalog.advance.resource';
const widgetMavenUtils = require('../utils/WidgetMavenUtils');
const juicer = require('juicer');
const fs = require('fs');

module.exports = {

     /**
     * 创建皮肤变量目录pId
     */
    createCatalogPId(v3cfg){
        let type = v3cfg.type;
        if(type=="resource"){//资源构件
            return "theme.catalog.base.resource";
        }else{
            let scope = v3cfg.scope;
            switch(scope){
                case "web":
                    return "theme.catalog.advance.pc.widget.$tag";
                case "mobile":
                    return "theme.catalog.advance.mobile.widget.$tag";
                default:
                    return "theme.catalog.base.tag";
            }
        }
    },

    /**
     * 创建皮肤变量目录Id
     */
    createCatalogId(v3cfg){
        let pId = this.createCatalogPId(v3cfg);
        return `${pId}.${v3cfg.pluginCode}`;
    },

    /**
     * 创建二次开发皮肤目录
     */
    createCatalogIPojoInst(v3cfg){
        return new Instance({
            component: 'theme.catalog',
            properties: {
                id: this.createCatalogId(v3cfg),
                pId: this.createCatalogPId(v3cfg),
                title: `${v3cfg.pluginCode}`
            }
        });
    },

    /**
     * 创建二次开发皮肤变量包
     * @param {Object} v3cfg 
     * @param {Instance} catalog
     * @param {Array} skinResPaths 
     */
    createSkinItemPackageIPojoInst(v3cfg, catalog,skinResPath){
        if (!skinResPath) {
            return;
        }
        const vjsName = widgetMavenUtils.getTagResourceVjsName();
        return new Instance({
            component: 'theme.varPackage',
            properties: {
                id: `${vjsName}.$lessVar`,
                catalogId: catalog.getProperty("id"),
                path: `${skinResPath}`
            }
        });
    },
    /**
     * 获取resources构件皮肤配置信息对应的ipojo信息
     * @param {Object} v3cfg v3devrc配置信息对象
     
    getSkinInstance(v3cfg) {
        const vjsName = widgetMavenUtils.getTagResourceVjsName();
        if (!v3cfg.skin) {
            return;
        }
        let schemes = v3cfg.skin.schemes;
        if (!schemes || schemes.length === 0) {
            return;
        }
        let ipojos = [];
        schemes.forEach(item => {
            let paths = item.paths.map(lessPath => {
                return widgetPathUtils.toPluginPagePath(path.basename(lessPath));
            });
            let instance = new Instance({
                component: 'theme.schemePackage',
                properties: {
                    id: `${vjsName}.${item.id}`,
                    interfaceVjsName: `${vjsName}.$skin`,
                    catalogId: `${idPrefix}.${v3cfg.pluginCode}`,
                    title: `${item.title}`,
                    desc: `${item.desc}`,
                    primaryColor: `${item.primaryColor}`,
                    paths: `${JSON.stringify(paths)}`
                }
            });
            ipojos.push(instance);
        });
        ipojos.push(new Instance({
            component: 'theme.catalog',
            properties: {
                id: `${idPrefix}.${v3cfg.pluginCode}`,
                pId: 'theme.catalog.advance.resource',
                title: `${v3cfg.pluginCode}`
            }
        }));
        return ipojos;
    },*/
    /**
     * 将解析到的皮肤变量信息生成ipojo对象字符串
     * @param {Object} v3cfg v3devrc配置信息对象
     * @param {Object} skinResPaths 皮肤保存文件路径
     
    getWigetSkinPojo(v3cfg, skinResPaths) {
        if (!skinResPaths || skinResPaths.length === 0) {
            return;
        }
        const vjsName = widgetMavenUtils.getTagResourceVjsName();
        let results = [];
        let instance = new Instance({
            component: 'theme.varPackage',
            properties: {
                id: `${vjsName}.$lessVar`,
                catalogId: this._getCatalog(v3cfg.scope),
                paths: `${JSON.stringify(skinResPaths)}`
            }
        });
        results.push(instance);
        return results;
    },*/
    /**
     * 根据项目类型获取catlogid
     * @param {String} scope 当前二次开发插件的scope配置信息
     
    _getCatalog(scope) {
        const prefix = 'theme.catalog.advance';
        const suffix = '.extension.tag';
        let catalog = '';
        switch (scope) {
            case 'web':
                catalog = prefix + '.pc' + suffix;
                break;
            case 'mobile':
                catalog = prefix + '.mobile' + suffix;
                break;
            default:
                catalog = prefix + suffix;
                break;
        }
        return catalog;
    },*/
    /**
     * 将解析到的皮肤变量信息输出为xml格式字符串
     * @param {Object} v3cfg v3devrc配置信息对象
     * @param {Object} skinVar 皮肤变量表
     */
    skinVarToXml(v3cfg, skinVar) {
        if (!skinVar) {
            return;
        }
        const vjsName = widgetMavenUtils.getTagResourceVjsName();
        let xmlObj = {
            define: {
                id: vjsName + '.$lessVar',
                catalogId: this.createCatalogId(v3cfg),
                belongToVjsName: widgetMavenUtils.getWidgetSkinVjsName(),
                items: {
                    item: []
                }
            }
        }
        Object.keys(skinVar).forEach(key => {
            xmlObj.define.items.item.push(skinVar[key]);
        });
        var builder = new xml2js.Builder({
            cdata: true
        });
        return builder.buildObject(xmlObj);
    },
    /**
     * 创建皮肤信息相关vjs
     * @param {Object} v3cfg v3devrc配置信息对象
     * @param {Object} skinInfo 皮肤信息
     */
    getSkinVjs(v3cfg, skinInfo) {
        if (!v3cfg || !skinInfo || !skinInfo.metadata) {
            return;
        }
        const vjsName = widgetMavenUtils.getTagResourceVjsName();
        let vjsList = [];
        let tmplPath = path.resolve(__dirname, 'skinVjsTmpl.tpl');
        let tmpStr = fs.readFileSync(tmplPath, 'utf8');
        let interfaceVjsName = vjsName + '.$lessVar';
        let prams = {
            interfaceVjsName: interfaceVjsName,
            lessScripts: skinInfo.metadata || [],
            deps: [interfaceVjsName, "vjs.framework.extension.platform.skin.base"]
        }
        prams.deps = JSON.stringify(prams.deps);
        prams.lessScripts = JSON.stringify(prams.lessScripts);
        let factoryStr = juicer(tmpStr, prams);
        let modules = [new Module({
            id: widgetMavenUtils.toModuleId(widgetMavenUtils.getTagResourceVjsName(), "skin", v3cfg.version),
            factory: factoryStr
        })];
        let vjs = new Vjs({
            name: widgetMavenUtils.getWidgetSkinVjsName(),
            dependencies: [
                new Dependency({
                    name: interfaceVjsName,
                    version: null,
                    optional: true
                }),
                new Dependency({
                    name: "vjs.framework.extension.platform.skin.base",
                    version: null
                })
            ],
            modules: modules
        });
        vjsList.push(vjs);
        return vjsList;
    }
}