/**
 * 皮肤资源构件ipojo实体生成
 * @author xuwj 2018-12-7
 */
const path = require('path');
const Instance = require('../jar/model/Instance');
const widgetPathUtils = require('./WidgetPathUtils');
const idPrefix = 'theme.catalog.advance.resource';
module.exports = {
    /**
     * 获取皮肤配置信息对应的ipojo信息
     * @param {Object} v3cfg v3devrc配置信息对象
     */
    getSkinInstance(v3cfg) {
        if (!v3cfg.skin) {
            return;
        }
        let schemes = v3cfg.skin.schemes;
        if (!schemes || schemes.length === 0) {
            return;
        }
        let ipojos = [];
        schemes.forEach(item => {
            let paths = item.paths.map(lessPath => {
                return widgetPathUtils.toPluginPagePath(path.basename(lessPath));
            });
            let instance = new Instance({
                component: 'theme.schemePackage',
                properties: {
                    id: `${v3cfg.VjsList[0].name}.${item.id}`,
                    interfaceVjsName: `${v3cfg.VjsList[0].name}.$skin`,
                    catalogId: `${idPrefix}.${v3cfg.pluginCode}`,
                    title: `${item.title}`,
                    desc: `${item.desc}`,
                    primaryColor: `${item.primaryColor}`,
                    paths: `${JSON.stringify(paths)}`
                }
            });
            ipojos.push(instance);
        });
        ipojos.push(new Instance({
            component: 'theme.catalog',
            properties: {
                id: `${idPrefix}.${v3cfg.pluginCode}`,
                pId: 'theme.catalog.advance.resource',
                title: `${v3cfg.pluginCode}`
            }
        }));
        return ipojos;
    },

    _getCatalog(scope) {
        const prefix = 'theme.catalog.advance';
        const suffix = '.extension.tag';
        let catalog = '';
        switch (scope) {
            case 'web':
                catalog = prefix + '.pc' + suffix;
                break;
            case 'mobile':
                catalog = prefix + '.mobile' + suffix;
                break;
            default:
                catalog = prefix + suffix;
                break;
        }
        return catalog;
    }
}