const pathUtils = require('../../utils/PathUtils');
const fileUtils = require('../../utils/FileUtils');
const constant = require('../../utils/Constant');
const configUtils = require('../../utils/ConfigUtils');
const widgetMavenUtils = require('./WidgetMavenUtils');
const path = require('path');

module.exports = {

    _getJarDir: function () {
        let v3DevCfg = configUtils.getV3DevConfig();
        let out = v3DevCfg.out ? v3DevCfg.out : constant.DEFAULT_PLUGIN_DIR;
        return pathUtils.toPluginPath(out);
    },

    getJarAbsPath: function (version) {
        if (!version) {
            version = configUtils.getPluginVersion();
        }
        return this._getJarDir() + '/' + widgetMavenUtils.getSymbolicName() + '-' + version + '.jar';
    },

    getSourceJarAbsPath: function (version) {
        if (!version) {
            version = configUtils.getPluginVersion();
        }
        return this._getJarDir() + '/' + widgetMavenUtils.getSymbolicName() + '-' + version + '-sources.jar';
    },

    /**
     * 获取控件vue单文件组件路径
     */
    getWidgetVuePath: function () {
        let relativeDir = "src/components/widgets";
        let pluginCode = configUtils.getPluginCode();
        return pathUtils.toPluginPath(path.join(relativeDir, pluginCode, "index.vue"));
    },

    toPluginTGZSettingPath: function (path) {
        let pluginType = configUtils.getPluginType();
        return constant.SETTING_PLUGIN_TGZ_DIR + pluginType + '/' + path;
    },

    getPluginTGzPath: function () {
        let packageJsonObj = configUtils.getPackageConfig();
        let name = packageJsonObj.name;
        let pluginPath = pathUtils.getPluginPath();
        let filePaths = fileUtils.listFiles(pluginPath, false, null);
        let result = [];
        if (filePaths && filePaths.length > 0) {
            filePaths.forEach(filePath => {
                let fileName = fileUtils.getFileName(filePath);
                if (fileName.startsWith(name) && fileName.endsWith(".tgz")) {
                    result.push(filePath);
                }
            });
        }
        return result;
    },

    toPluginPagePath: function (path) {
        return constant.PAGE_RESOURCE_DIR + configUtils.getPluginType() + '/' + configUtils.getPluginCode() + '/' + path;
    }
};