const WidgetPluginFactory = require('./widget/WidgetPluginFactory');
const ResourcePluginFactory = require('./resource/ResourcePluginFactory');
const WindowPluginFactory = require('./window/WindowPluginFactory');
const ResourceWebpackCfgGenerator = require('./resource/webpack/ResourceWebpackCfgGenerator');
const WidgetWebpackCfgGenerator = require('./widget/webpack/WidgetWebpackCfgGenerator');
const configUtils = require('../utils/ConfigUtils');
const plType = require('../model/PluginType');
const v3Installer = require('v3-installer');
const pathUtils = require('../utils/PathUtils');
const fileUtils = require('../utils/FileUtils');
const path = require('path');
const updateUtils = require('v3-cli/src/utils/UpdateUtils');
module.exports = {

    setPackageConfigs: function () {
        return new Promise((resolve, reject) => {
            let pluginType = configUtils.getPluginType();
            let pluginPath = pathUtils.getPluginPath();
            // 先清除build文件夹
            fileUtils.removeDirectory(path.resolve(pluginPath, "build"));
            switch (pluginType) {
                case plType.resource:
                    /* 资源构件操作 */
                    ResourcePluginFactory.setPackageConfigs().then(() => {
                        resolve();
                    }).catch(err => {
                        reject(err);
                    });
                    break;
                case plType.widget:
                    let promise = WidgetPluginFactory.setPackageConfigs();
                    promise.then(() => {
                        resolve();
                    }).catch(err => {
                        reject(err);
                    });
                    break;
                default:
                    reject(new Error("暂时不支持【" + pluginType + "】二次开发，敬请期待！"));
            }
        });
    },
    /* 命令执行的入口 */
    exe: function (command, args) {
        if (command == 'install') {
            v3Installer._install(args);
            return;
        } else if (command == 'uninstall') {
            v3Installer._uninstall(args);
            return;
        }
        let exePromise;
        let pluginType = configUtils.getPluginType();
        switch (pluginType) {
            case plType.resource:
                exePromise = ResourcePluginFactory.exe(command, args);
                break;
            case plType.widget:
                exePromise = WidgetPluginFactory.exe(command, args);
                break;
            case plType.windwow:
                exePromise = WindowPluginFactory.exe(command, args);
                break;
            default:
                throw new Error("暂时不支持【" + pluginType + "】二次开发，敬请期待！")
        }
        exePromise && exePromise.then(() => {
            this.afterExe();
        }).catch(err => {
            console.log(err);
            // 如果命令执行过程中产生错误则尝试执行还原
            // BackupsUtil.restoreLess().then(() => {
            //     console.log(err.message || err);
            // }).catch(err => {
            //     console.log(err);
            // });
        });
    },
    // 命令执行完成后进行插件更新检查
    afterExe: function () {
        updateUtils.automaticUpdate().catch(err => {
            console.log(err);
        })
    },
    updateWebpackCfg: function (pluginType) {
        switch (pluginType) {
            case plType.widget:
                return new WidgetWebpackCfgGenerator().update();
            case plType.resource:
                return new ResourceWebpackCfgGenerator().update();
        }
    }
};