/**
 * 插件的命令对应的函数
 */
const configUtils = require('../../utils/ConfigUtils');
const pathUtils = require('../../utils/PathUtils');
const path = require('path');
const fs = require('fs-extra');
const engineFactory = require('../../packer/jar/engine/EngineFactory');
const Webpacker = require('../../utils/Webpacker');
const WebpackServer = require('../../utils/WebpackServer');
const plType = require('../../model/PluginType');
const cmdlist = require('./CommandList');
const npmRun = require('npm-run');
const webpackCfgUpdate = require('../../utils/webpackCfgUpdate');
const fileUtils = require('../../utils/FileUtils');
let skinInfo = undefined;
/* 全部函数列表 */
const funList = {
    _pack: function (args) {
        return new Promise((resolve, reject) => {
            let promise = this._webpack(args);
            promise.then(() => {
                console.log('正在生成VStore构件，请稍候...');
                let engine = engineFactory.createEngine("install");
                let context = engine.getContext();
                context.put("skinInfo",skinInfo);
                context.put("args",args);
                return engine.start();
            }).then(() => {
                console.log('VStore构件生成成功！');
                resolve();
            }).catch(err => {
                console.error('VStore插件生成出现错误！');
                reject(err);
            });
        });
    },

    _apply: function (args) {
        return new Promise((resolve, reject) => {
            let promise = this._webpack();
            promise.then(() => {
                console.log('正在将V3平台插件安装到执行系统中，请稍候...');
                let engine = engineFactory.createEngine("apply");
                let context = engine.getContext();
                context.put("skinInfo",skinInfo);
                context.put("args",args);
                return engine.start();
            }).then(() => {
                console.log('V3平台插件安装到执行系统完成！');
                resolve();
            }).catch(err => {
                console.error('V3平台插件安装到执行系统出现错误！');
                reject(err);
            });
        });
    },

    _publish: function (args) {
        return new Promise((resolve, reject) => {
            let promise = this._webpack();
            promise.then(() => {
                console.log('正在将V3平台插件部署到VStore仓库，请稍候...');
                let engine = engineFactory.createEngine("deploy");
                let context = engine.getContext();
                context.put("skinInfo",skinInfo);
                context.put("args",args);
                return engine.start();
            }).then(() => {
                console.log('部署插件到VStore仓库完成！');
                resolve();
            }).catch(err => {
                console.error('部署插件到VStore仓库出现错误！');
                reject(err);
            });

        });
    },
    _webpack: async function (args) {
        let webpackCfgPath = path.resolve(pathUtils.getPluginPath(), "build/webpack.config.js");
        try {
            await webpackCfgUpdate.checkWebpackCfgUpdate();
            console.log('正在使用webpack打包控件，请稍候...');
            let v3DevConfig = configUtils.getV3DevConfig();
            let sourcePath = path.resolve(pathUtils.getPluginPath(), v3DevConfig.sources);
            if (fs.existsSync(sourcePath)) {
                fs.removeSync(sourcePath);
            }
            let message = await new Webpacker(webpackCfgPath).pack();
            if (configUtils.enableThemeLoader()) {
                skinInfo = await this._postPack();
            }
            console.log('webpack打包控件完成！');
            return message;
        } catch (error) {
            throw error;
        }
    },
    /**
     * 
     * 打包执行前的预处理
     */
    _prePack: async function (webpackCfgPath) {},
    /**
     * 打包完成后的处理
     */
    _postPack: async function () {
        const vthemeOption = require('vtheme-loader/src/model/Options');
        const postHandler = require('vtheme-loader/src/postHandler/PostHandler');
        let v3DevConfig = configUtils.getV3DevConfig();
        let distCssPath = path.resolve(pathUtils.getPluginPath(), `${v3DevConfig.sources}/${v3DevConfig.type}/${v3DevConfig.pluginCode}.css`);
        let tmpPath = path.resolve(pathUtils.getPluginPath(), vthemeOption.varsBaseDir);
        let skinVars = [],
            metadata = [];
        try {
            if (fs.existsSync(distCssPath)) {
                let cssContent = fs.readFileSync(distCssPath, 'utf8');
                let result = await postHandler.handle(cssContent, distCssPath, true);
                let {
                    scopedStyle,
                    globalStyle
                } = result;
                if (scopedStyle) {
                    for (const item of scopedStyle) {
                        metadata.push({
                            scopedId: item.scopedId,
                            less: item.less
                        });
                        skinVars = skinVars.concat(item.skinVars || []);
                    }
                }
                if (globalStyle && globalStyle.less && globalStyle.less.trim().length > 0) {
                    metadata.push({
                        less: globalStyle.less
                    });
                    skinVars = skinVars.concat(globalStyle.skinVars || []);
                }
            }
            fileUtils.removeDirectory(tmpPath);
            return {
                metadata: metadata,
                skinVars: skinVars
            }
        } catch (error) {
            fileUtils.removeDirectory(tmpPath);
            throw new Error(`${error},样式文件${distCssPath}渲染失败！`);
        }
    },
    _test: function () {
        return new Promise((resolve, reject) => {
            console.log('正在启动本地测试服务，请稍候...');
            configUtils.setTestServerEnv(true);
            let server = new WebpackServer(path.resolve(pathUtils.getPluginPath(), "build/webpack.config.js"));
            let promise = server.start();
            promise.then(() => {
                return resolve();
            }).catch(err => {
                configUtils.setTestServerEnv(false);
                reject(err);
            });
        });
    },

    window_webpack: function () {
        return new Promise((resolve, reject) => {
            console.log("开始使用webpack打包二次开发窗体，请稍候...");
            try {
                let configs = require("../window/webpack/build/webpack.prod.config");
                const webpack = require('webpack');
                const ProgressPlugin = require('webpack/lib/ProgressPlugin');
                const compiler = webpack(configs);
                let prePercentage = 0;
                compiler.apply(new ProgressPlugin((percentage, msg, current, active, modulepath) => {
                    percentage = parseInt((percentage * 100).toFixed(0));
                    if (percentage > prePercentage) {
                        prePercentage = percentage;
                        console.log(prePercentage + '%', msg);
                    }
                }));
                compiler.run(function (err, stats) {
                    let message = stats.toString({
                        hash: false,
                        modules: false,
                        children: false,
                        colors: true
                    });
                    if (err || stats.hasErrors()) {
                        reject(message);
                        return;
                    }
                    console.log('二次开发窗体打包完成！');
                    resolve();
                });
            } catch (err) {
                reject(err);
            }
        });
    },

    window_test: function () {
        return new Promise((resolve, reject) => {
            console.log("开始启动本地测试服务，请稍候...");
            npmRun.exec('cross-env NODE_ENV=development webpack-dev-server --config ' + pathUtils.getSelfPath() + '/src/plugin/window/webpack/build/webpack.dev.config.js', {
                cwd: pathUtils.getPluginPath()
            }, (err, stdout, stderr) => {
                if (err) {
                    reject(err)
                    return;
                }
                console.log('本地测试服务完成！');
                resolve();
            });
        });
    },
    notSupport: function (pluginType, cmd) {
        return new Promise((resolve, reject) => {
            reject(plType.getAllTips[pluginType] + '开发暂不支持命令【' + cmd + '】');
        });
    }
}

module.exports = {

    /**
     * 根据插件命令和插件类型获取处理函数
     * @param {String} cmd 插件命令
     * @param {String} pluginType 插件类型
     */
    getFunction(cmd, pluginType,args) {
        switch (pluginType) {
            case plType.widget:
                return this.getWidgetFunc(cmd,args);
            case plType.resource:
                return this.getResourceFunc(cmd,args);
            case plType.window:
                return this.getWindowFunc(cmd,args);
            default:
                return new Promise((resolve, reject) => {
                    reject('不支持的插件类型: ' + plType);
                });
        }
    },
    /**
     * 获取窗体插件命令函数
     * @param {String} cmd 插件命令
     */
    getWindowFunc(cmd) {
        switch (cmd) {
            case cmdlist.webpack:
                return funList.window_webpack(args);
            case cmdlist.start:
                return funList.window_test(args);
            default:
                return funList.notSupport(plType.window, cmd);
        }
    },
    /**
     * 获取控件插件命令函数
     * @param {String} cmd 插件命令
     */
    getWidgetFunc(cmd,args) {
        switch (cmd) {
            case cmdlist.pack:
                return funList._pack(args);
            case cmdlist.publish:
                return funList._publish(args);
            case cmdlist.apply:
                return funList._apply(args);
            case cmdlist.webpack:
                return funList._webpack(args);
            case cmdlist.test:
            case cmdlist.start:
                return funList._test(args);
            default:
                return funList.notSupport(plType.widget, cmd);
        }
    },
    /**
     * 获取资源插件命令函数
     * @param {String} cmd 插件命令
     */
    getResourceFunc(cmd,args) {
        switch (cmd) {
            case cmdlist.pack:
                return funList._pack(args);
            case cmdlist.publish:
                return funList._publish(args);
            case cmdlist.apply:
                return funList._apply(args);
            case cmdlist.webpack:
                return funList._webpack(args);
            case cmdlist.test:
            case cmdlist.start:
                return funList._test(args);
            default:
                return funList.notSupport(plType.resource, cmd);
        }
    }
}