/**
 * 插件命令基类
 */

const commandFunctions = require('./CommandFunctions');
class PluginCommand {
    constructor() {}
    /**
     * 获取全部支持的命令,子类必须覆盖该方法
     * @abstract
     */
    getAllCommand() {
        return [];
    }
    /**
     * 获取插件类型,子类必须覆盖该方法以表明自身支持的插件类型
     * @abstract
     */
    getPluginType() {
        return null;
    }
    /**
     * 判断一个命令是否受当前插件类型支持
     * @param cmdCheck 要检查的命令名
     */
    isSupport(cmdCheck) {
        this.getAllCommand().forEach(cmd => {
            if (cmd == cmdCheck) {
                return true;
            }
        });
        return false;
    }
    /**
     * 根据插件类型和命令返回对应于当前插件命令的函数
     * @param {String} command 要获取函数的对应命令
     */
    getCmdFunction(command,args) {
        return commandFunctions.getFunction(command, this.getPluginType(),args);
    }
}

module.exports = PluginCommand;