const pluginPathUtils = require('../utils/PathUtils');
const pathUtils = require('../../../utils/PathUtils');
const path = require("path");
const fs = require("fs");
const vueTempCompiler = require("vue-template-compiler");
const VueComponentAnalyze = require("../../../analyzer/VueComponentAnalyzer");
const ioUtils = require("../../../utils/IOUtils");

let MetadataGenerator = function(components){
    this.components = components;
};

MetadataGenerator.prototype = {

    generate : function(){
        this.components.forEach(component => {
            let componentCode = component.getCode();
            let windows = component.getWindows();
            if(windows){
                windows.forEach(win => {
                    let windowCode = win.getCode();
                    let widgets = win.getWidgets();
                    if(widgets){
                        widgets.forEach(widget => {
                            let widgetCode = widget.getCode();
                            let relativePath = pluginPathUtils.getWidgetIndexVuePath(componentCode,windowCode,widgetCode);
                            let absPath = path.join(pathUtils.getPluginPath(),relativePath);
                            fs.readFile(absPath, 'utf8', (function(cmCode,wnCode,wdCode){
                                return function(err,data){
                                    let rs = vueTempCompiler.parseComponent(data);
                                    let javascript = "";
                                    let domScript = "";
                                    if(rs.script){
                                        javascript = rs.script.content;
                                    }
                                    if(rs.template){
                                        domScript = rs.template.content;
                                    }
                                    let analyzer = new VueComponentAnalyze({
                                        domScript : domScript,
                                        javascript : javascript
                                    });
                                    let savePath = pluginPathUtils.toWidgetMetadataPath(cmCode,wnCode,wdCode);
                                    let cmp = analyzer.getComponent();
                                    let metadata = {
                                        events : cmp.getEvents(),
                                        tagnames : cmp.getTagNames()
                                    };
                                    ioUtils.writeContentAsync(pathUtils.getPluginPath()+"/dist/"+savePath,JSON.stringify(metadata,null,"\t"),function(err){
                                        if(err){
                                            throw err;
                                        }
                                    });
                                };
                            })(componentCode,windowCode,widgetCode));
                        });
                    }
                });
            }
        });
    }
};

module.exports = MetadataGenerator;