const fs = require("fs");
const path = require("path");
const pathUtils = require("../../../utils/PathUtils");
const Component = require("../model/Component");
const Window = require("../model/Window");
const Widget = require("../model/Widget");
const pluginPathUtils = require('./PathUtils');
const metadataPath = "src/components/metadata.json";

module.exports = {

  /**
   * 分析构件，获取构件、窗体信息
   */
  analyzeComponents: function () {
    let pluginPath = pathUtils.getPluginPath();
    let metdataAbsPath = path.join(pluginPath, metadataPath);
    try {
      let results = [];
      let metdataStr = fs.readFileSync(metdataAbsPath);
      let metadata = JSON.parse(metdataStr);
      metadata.forEach(cmp => {
        let componentCode = cmp.code;
        let windows = [];
        let wins = cmp.windows;
        if (wins) {
          wins.forEach(win => {
            let windowCode = win.code;
            let widgets = [];
            let wds = win.widgets;
            if (wds) {
              wds.forEach(wd => {
                widgets.push(new Widget({
                  code: wd.code
                }));
              });
            }
            windows.push(new Window({
              code: windowCode,
              name: win.name,
              widgets: widgets,
              indexPath: "./" + pluginPathUtils.getWindowIndexPath(componentCode, windowCode),
              previewIndexPath: "./" + pluginPathUtils.getWindowPreviewIndexPath(componentCode, windowCode)
            }));
          });
        }
        results.push(new Component({
          code: componentCode,
          name: cmp.name,
          srcPath: pluginPathUtils.getComponentSrcPath(componentCode),
          windows: windows
        }));
      });
      return results;
    } catch (e) {
      throw Error("解析json文件失败，请检查文件内容！错误原因：" + e.message + "，文件路径：" + metadataPath, e);
    }
  }

};