const componentUtils = require('../../utils/ComponentUtils');
let components = componentUtils.analyzeComponents();

let rules = require("../config/WebpackDefaultLoaders");
rules = rules.concat((require("../config/WebpackFileLoader").getLoaders(components)));

const Generator = require("../config/WindowPreviewIndexGenerator");
let generator = new Generator(components);
generator.generate();

const merge = require("webpack-merge");
let baseConfig = require("../config/WebpackBaseConfig");

const externals = require('../config/WebpackExternals');

let defaultAlias = require("../config/WebpackDefaultAlias");
let componentAlias = require("../config/WebpackComponentAlias").getComponentAlias(components);
let alias = merge(defaultAlias, componentAlias);

let defaultPlugins = require("../config/WebpackDefaultPlugin").getPlugins(components);
let htmlPlugins = require("../config/WebpackHtmlPlugin").getPlugins(components);
let extractTextPlugins = require("../config/WebpackExtractTextPlugin").getPlugins(components);
let plugins = defaultPlugins.concat(extractTextPlugins).concat(htmlPlugins);

let defaultEntry = require("../config/WebpackDefaultEntry");
let componentEntry = require("../config/WebpackComponentEntry").getEntries(components);
let entry = merge(defaultEntry, componentEntry);

const fileUtils = require('../../../../utils/PathUtils');
const path = require('path');

let webpackConfigs = {
  externals: externals,
  module: {
    rules: rules
  },
  entry: entry,
  context: path.resolve(fileUtils.getPluginPath()),
  resolve: {
    alias: alias
  },
  plugins: plugins
};

let configs = merge(baseConfig, webpackConfigs);

module.exports = {
  configs: configs,
  components: components
};