const fs = require("fs");
const path = require('path');
const arrayUtils = require("./ArrayUtils");
const pathUtils = require("./PathUtils");
const PathSep = path.sep;
module.exports = {

	listFiles: function (dirPath, recursion, excludes) {
		recursion = typeof (recursion) == 'boolean' ? recursion : false;
		if (!fs.existsSync(dirPath)) {
			throw Error("文件路径不存在，请检查！path=" + dirPath);
		}
		let absPaths = [];
		if (this.isDirectory(dirPath)) {
			let fileNames = fs.readdirSync(dirPath);
			if (fileNames && fileNames.length > 0) {
				for (let j = 0, len = fileNames.length; j < len; j++) {
					let absPath = dirPath + PathSep + fileNames[j];
					if (this.isFile(absPath) && (!excludes || !arrayUtils.contains(excludes, absPath))) {
						absPaths.push(absPath);
					}
					if (recursion) {
						if (this.isDirectory(absPath) && !arrayUtils.contains(excludes, absPath)) {
							absPaths = absPaths.concat(this.listFiles(absPath, recursion, excludes));
						}
					}
				}
			}
		} else {
			throw Error("路径错误，不是文件夹！path=" + dirPath);
		}
		return absPaths;
	},

	isDirectory: function (path) {
		if (fs.existsSync(path)) {
			let stat = fs.statSync(path);
			return stat.isDirectory();
		}
		return false;
	},

	isFile: function (path) {
		if (fs.existsSync(path)) {
			let stat = fs.statSync(path);
			return stat.isFile();
		}
		return false;
	},

	filte: function (files, subfix) {
		let rs = [];
		if (!files) return rs;
		for (let i = 0, l = files.length; i < l; i++) {
			let path = files[i];
			if (path.endsWith(subfix)) {
				rs.push(path);
			}
		}
		return rs;
	},

	removeDuplicateDir: function (files) {
		var rs = {};
		var temp = [];
		for (let i = 0, l = files.length; i < l; i++) {
			var path = files[i];
			var arr = this._pathToArray(path);
			arr.orginal = path;
			temp.push(arr);
		}
		var parentArray, index = -1;
		for (let i = 0, l = temp.length; i < l; i++) {
			var array = temp[i];
			if (!parentArray) {
				parentArray = array.concat();
				continue;
			} else {
				for (var j = 0; j < parentArray.length; j++) {
					if (parentArray[j] != array[j]) {
						index = j - 1;
						parentArray.length = j;
						break;
					}
				}
			}
		}
		if (temp.length == 1) {
			let arr = temp[0];
			rs[arr.orginal] = arr.pop();
		} else {
			for (let i = 0, l = temp.length; i < l; i++) {
				let arr = temp[i];
				rs[arr.orginal] = arr.slice(index + 1).join(PathSep);
			}
		}
		return rs;
	},

	_pathToArray: function (p) {
		let rs = [];
		while (true) {
			let basename = path.basename(p);
			if (basename) {
				rs.push(basename);
				p = path.resolve(p, "..");
			} else {
				rs.push(p);
				break;
			}
		}
		return rs.reverse();
	},

	makeDir: function (filePath) {
		let iter = function (parent) {
			if (!fs.existsSync(parent)) {
				let f = pathUtils.getParentPath(parent);
				iter(f);
				fs.mkdirSync(parent);
			}
		};
		let parent = pathUtils.getParentPath(filePath);
		iter(parent);
		if (!fs.existsSync(filePath)) {
			fs.mkdirSync(filePath);
		}
	},

	getFileName: function (p) {
		return path.basename(p);
	},

	removeDirectory: function (dirPath) {
		try {
			if (this.isDirectory(dirPath)) {
				let fileNames = fs.readdirSync(dirPath);
				if (fileNames && fileNames.length > 0) {
					for (let j = 0, len = fileNames.length; j < len; j++) {
						let absPath = dirPath + PathSep + fileNames[j];
						if (this.isDirectory(absPath)) {
							this.removeDirectory(absPath);
						} else {
							fs.unlinkSync(absPath);
						}
					}
				}
				fs.rmdirSync(dirPath);
			}
		} catch (err) {
			console.log('目录删除失败!');
			console.log(err);
		}
	},

	getV3PluignTempRelativeDir: function () {
		let relativePath = "temp";
		do {
			relativePath += "_";
			let absPath = pathUtils.toPluginPath(relativePath);
		} while (fs.existsSync(absPath));
		return relativePath;
	},

	toV3PluginTempRelativePath: function (relativePath) {
		return PathSep + relativePath + PathSep + "plugins.zip";
	}

};