const fs = require("fs");
const objectUtils = require("./ObjectUtils");
const path = require('path');
const constant = require("./Constant");
const _thisDir = __dirname;
const PathSep = path.sep;
let matchPaths = {
	"bin": true,
	"bundle": {
		"drivers": true,
		"extensions": {
			"dropins": true,
			"vplatform": true
		},
		"system": true
	},
	"cache": true,
	"conf": {
		"setting": true
	},
	"pages": true
};

module.exports = {


	getSelfPath: function () {
		return path.resolve(_thisDir, '../../');
	},

	getPluginPath: function () {
		let pathParams = fs.realpathSync('.');
		let index = pathParams.indexOf("node_modules");
		if (index != -1) {
			pathParams = pathParams.substring(0, index);
		}
		return pathParams;
	},

	toPluginPath: function (pathParams) {
		return path.resolve(this.getPluginPath(), pathParams);
	},

	getParentPath: function (pathParams) {
		return path.resolve(pathParams, "..");
	},

	contains: function (source, compare) {
		var founded = true;
		for (var i = 0, l = compare.length; i < l; i++) {
			var index = source.indexOf(compare[i]);
			if (index == -1) {
				founded = false;
				break;
			}
		}
		return founded;
	},

	listKeys: function (obj) {
		var keys = [];
		for (var attr in obj) {
			keys.push(attr);
		}
		return keys;
	},

	isMatchPath: function (pathParams, matcher) {
		matcher = matcher || matchPaths;
		var names = this.listFileNames(pathParams);
		var keys = this.listKeys(matcher);
		var result = true;
		if (contains(names, keys)) {
			for (var key in matcher) {
				var val = matcher[key];
				if (objectUtils.isObject(val)) {
					result = result && this.isMatchPath(pathParams + PathSep + key, val);
				}
			}
		} else {
			result = false;
		}
		return result;
	},

	//获取执行系统根目录
	getRuntimeDir: function () {
		var pathParams = this.getPluginPath();
		while (true) {
			if (!this.isMatchPath(pathParams, null)) {
				var path1 = this.getParentPath(pathParams);
				if (path1 != pathParams) {
					pathParams = path1;
				} else {
					pathParams = null;
					break;
				}
			} else {
				break;
			}
		}
		return pathParams;
	},

	getV3PluignTempRelativeDir: function () {
		var relativePath = "temp";
		do {
			relativePath += "_";
			let absPath = this.toPluginPath(relativePath);
		} while (fs.existsSync(absPath));
		return relativePath;
	},

	getV3PluginTGZDir: function () {
		var pathParams = environment.getPluginTempPath();
		return pathParams + PathSep + constant.SETTING_PLUGIN_TGZ_DIR + environment.getPluginName();
	}
};