const juicer = require('juicer');
const fs = require('fs');
const ioUtils = require('./IOUtils');
const path = require('path');
const pathUtils = require('./PathUtils');
const configUtils = require('./ConfigUtils');
let v3WebpackCfg = null;
/**
 * 获取模板文件内容
 * @param {String} absPath 模板路径
 */
function getContentFromTpl(absPath, extData) {
    return new Promise((resolve, reject) => {
        if (!path.isAbsolute(absPath)) {
            reject(new Error('webpack配置模板文件加载失败!'));
            return;
        }
        fs.readFile(absPath, (err, content) => {
            if (err) {
                reject(err);
                return;
            }
            // 修改juicer语法边界,消除语法冲突导致配置文件生成失败!
            juicer.set({
                'strip': false,
                'cache': false,
                'tag::interpolateOpen': '$j{',
            });
            let cfg = configUtils.getV3DevConfig();
            if (extData) {
                Object.assign(cfg, extData);
            }
            resolve(juicer(String(content), {
                config: cfg
            }));
        });
    });
}

module.exports = {
    /**
     * 创建文件并写入数据
     * @param {String} tplPath 模板文件的绝对路径
     * @param {String} savePath 存储路径,相对用户项目的路径
     * @param {Object} extData 额外的模板数据对象
     * @returns Promise
     */
    createFile(tplAbsPath, savePath, extData) {
        return new Promise((resolve, reject) => {
            let promise = getContentFromTpl(tplAbsPath, extData);
            promise.then(content => {
                return ioUtils.write(path.resolve(pathUtils.getPluginPath(), savePath), content);
            }).then(() => {
                resolve();
            }).catch(err => {
                reject(err);
            });
        });
    },
    /**
     * 获取用户项目的webpack配置文件
     */
    getV3WebpackCfg: function () {
        if (!v3WebpackCfg) {
            let cfgPath = path.resolve(pathUtils.getPluginPath(), 'build', 'v3', 'webpack.v3.config.js');
            try {
                v3WebpackCfg = require(cfgPath);
            } catch (err) {
                v3WebpackCfg = null;
            }
        }
        return v3WebpackCfg;
    }
}