const Webpack = require('webpack');
const WebpackDevServer = require('webpack-dev-server/lib/Server');
const ProgressPlugin = require('webpack/lib/ProgressPlugin');
const detectPort = require('detect-port');
const open = require('opn');

class WebpackServer {

    constructor(configPath) {
        this.configPath = configPath;
    }

    start() {
        return new Promise((resolve, reject) => {
            try {
                let webpackConfig = require(this.configPath);
                const compiler = Webpack(webpackConfig);
                let prePercentage = 0;
                compiler.apply(new ProgressPlugin((percentage, msg, current, active, modulepath) => {
                    percentage = parseInt((percentage * 100).toFixed(0));
                    if(percentage>prePercentage){
                        prePercentage = percentage;
                        console.log( prePercentage+ '%', msg);
                    }
                }));
                const devServerOptions = Object.assign({}, webpackConfig.devServer||{}, {
                    stats: {
                        colors: true
                    }
                });
                const server = new WebpackDevServer(compiler, devServerOptions);
                this._getValidPort().then(port=>{
                    server.listen(port, '127.0.0.1', () => {
                        console.log('Starting server on http://localhost:'+port);
                        open("http://localhost:"+port+"/index.html");
                    });
                }).catch(err=>{
                    reject(err);
                });
            } catch (err) {
                reject(err);
            }
        });
    }

    _getValidPort() {
        return new Promise((resolve, reject) => {
            let port = 7800;
            detectPort(port).then(pt=>{
               resolve(pt);
            }).catch(err=>{
                reject(err);
            });
        });
    }

}

module.exports = WebpackServer;