const webpack = require('webpack');
const ProgressPlugin = require('webpack/lib/ProgressPlugin');
const pathUtils = require('./PathUtils');
const path = require('path');
const fs = require('fs-extra');
const configUtils = require('../utils/ConfigUtils');
const fileUtils = require('../utils/FileUtils');
class Webpacker {

    constructor(configPath) {
        this.configPath = configPath;
    }

    pack() {
        return new Promise((resolve, reject) => {
            try {
                let configs = require(this.configPath);
                //优化处理，如果入口文件大小为0，则忽略webpack打包
                let needPack = false;
                let contextPath = configs.context || pathUtils.getPluginPath();
                for (let [key, entry] of Object.entries(configs.entry)) {
                    let absPath = path.resolve(contextPath, entry);
                    let status = fs.statSync(absPath);
                    if (status.size > 0) {
                        needPack = true;
                        break;
                    }
                }
                let v3Cfg = configUtils.getV3DevConfig();
                let urlSources = v3Cfg.urlSources;
                // 是否拷贝外部资源
                let copyExt = this.needCopyExtr(v3Cfg);
                let hasUrlsources = urlSources && urlSources.length > 0 ? true : false;
                //入口文件没有内容，忽略webpack打包
                if (!needPack && copyExt) {
                    /*this.copyExtraLib(v3Cfg).then(() => {
                        resolve();
                        return;
                    }).catch(err => {
                        reject(err);
                        return;
                    });*/
                    return resolve();
                } else if (!needPack && !copyExt && !hasUrlsources) {
                    //入口文件没有内容,又没有配置extraSources,urlSources则拒绝打包
                    return reject('请勿打包空项目!');
                } else if (needPack && copyExt) {
                    //入口文件有内容又配置了extraSources
                    return reject('已配置extraSources属性,请清空index.js文件内容!');
                } else if (needPack && !copyExt) {
                    const compiler = webpack(configs);
                    let prePercentage = 0;
                    compiler.apply(new ProgressPlugin((percentage, msg, current, active, modulepath) => {
                        percentage = parseInt((percentage * 100).toFixed(0));
                        if (percentage > prePercentage) {
                            prePercentage = percentage;
                            console.log(prePercentage + '%', msg);
                        }
                    }));
                    compiler.run(function (err, stats) {
                        let message = stats.toString({
                            hash: false,
                            modules: false,
                            children: false,
                            colors: true
                        });
                        if (err || stats.hasErrors()) {
                            reject(message);
                            return;
                        }
                        resolve(message);
                    });
                }else{
                    resolve();
                }
            } catch (err) {
                reject(err);
            }
        });
    }
    /**
     * 判断是否需要拷贝外部资源,配置了extraSources项或skin项都需要拷贝
     */
    needCopyExtr(v3Cfg) {
        let extraSources = v3Cfg.extraSources;
        let skinCfg = v3Cfg.skin;
        if (extraSources && extraSources.length > 0) {
            return true;
        }
        if (skinCfg && skinCfg.schemes) {
            return true;
        }
        return false;
    }

    copyExtraLib(v3Cfg) {
        return new Promise((resolve, reject) => {
            let copyFiles = v3Cfg.extraSources || [];
            let skinCfg = v3Cfg.skin;
            if (skinCfg && skinCfg.schemes) {
                skinCfg.schemes.forEach(item => {
                    let lessPath = item.paths;
                    if (lessPath && lessPath.length > 0) {
                        copyFiles = copyFiles.concat(lessPath);
                    }
                });
            }

            // 获取打包输出路径
            let distPath = path.resolve(pathUtils.getPluginPath(), v3Cfg.sources, v3Cfg.type);
            fileUtils.removeDirectory(distPath);
            // 拷贝全部外部lib到输出目录,文件则直接拷贝，文件夹则拷贝文件夹内全部文件
            copyFiles.forEach(libItem => {
                let outPath = distPath;
                if (fileUtils.isFile(libItem)) {
                    outPath = path.resolve(distPath, path.basename(libItem));
                }
                if (fs.existsSync(libItem)) {
                    try {
                        fs.copySync(libItem, outPath);
                    } catch (err) {
                        reject(err);
                    }
                }
            });
            resolve();
        });
    }
}

module.exports = Webpacker;