const WidgetWebpackCfgGenerator = require('../plugin/widget/webpack/WidgetWebpackCfgGenerator');
const ResourceWebpackCfgGenerator = require('../plugin/resource/webpack/ResourceWebpackCfgGenerator');
const pluginType = require('../model/PluginType');
const configUtils = require('./ConfigUtils');
module.exports = {
    /**
     * 获取webpack配置模板信息
     */
    getwebpackTmplMetadata: function () {
        let metadata;
        try {
            metadata = require('../plugin/webpackTmplMetadata.json');
        } catch (err) {
            console.log('webpack模板信息读取失败!');
        }
        return metadata;
    },
    /**
     * 检查用户项目的webpack配置文件是否需要更新
     */
    checkWebpackCfgUpdate: function () {
        return new Promise((resolve, reject) => {
            let v3DevConfig = configUtils.getV3DevConfig();
            let metadata = this.getwebpackTmplMetadata();
            if (v3DevConfig && metadata) {
                // 如果没有webpackCfgVersion属性则默认为1.0.0版本
                let userCfgVersion = v3DevConfig.webpackCfgVersion || '1.0.0';
                let newCfgVersion;
                metadata.forEach(cfg => {
                    cfg.pluginType == v3DevConfig.type ? newCfgVersion = cfg.webpackCfgVersion : null;
                });
                if (!newCfgVersion) {
                    reject(`没有找到插件类型为${v3DevConfig.type}的webpack配置模板信息!`);
                    return;
                } else if (newCfgVersion > userCfgVersion) {
                    console.log('检测到webpack配置文件有更新,开始更新webpack配置文件....');
                    let updatePromsie;
                    switch (v3DevConfig.type) {
                        case pluginType.widget:
                            updatePromsie = new WidgetWebpackCfgGenerator();
                            break;
                        case pluginType.resource:
                            updatePromsie = new ResourceWebpackCfgGenerator();
                            break;
                    }
                    if (updatePromsie) {
                        updatePromsie.update().then(() => {
                            v3DevConfig.webpackCfgVersion = newCfgVersion;
                            configUtils.persistenceV3DevCfg();
                            console.log('webpack配置文件已更新.');
                            resolve();
                        }).catch(err => {
                            console.log('webpack配置文件更新失败!');
                            reject(err);
                        })
                    }
                } else {
                    resolve();
                }
            } else {
                reject();
            }
        });
    }
}